#define SIZE 21
#define NSTEP 3000
#define NCELLTOT 200
#define NNEIGHBOR 3
#define NTRIES 1
#define  CONCENTRATION_SCALE 0.000000
#define NFREE_PRMT 0
static double free_prmt[] = {}; 
#define	NOUTPUT 4
#define	NINPUT 3
#define	NLIGAND 0
#define	NDIFFUSIBLE 4
#define SEED 28573
#define PRINT_BUF 1
#define DT 0.050000
static int trackin[] = {0, 6, 7};
static int trackout[] = {16, 18, 19, 20};
static int tracklig[] = {};
static int trackdiff[] = {16, 18, 19, 20};
static double diff_constant[] = {3.0, 3.0, 1.0, 1.0};
static int externallig[] = {};


 
/********  header file, arrays and functions common to all following subroutines and
  the same for all problems.  NOTE this file preceeded by compiler directives defining
  parameters and the track*[] arrays
********/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <limits.h>

/* global arrays for history and geometry */

static double history[SIZE][NSTEP][NCELLTOT];
static int geometry[NCELLTOT][NNEIGHBOR];

double MAX(double a,double b){
	 if (a>b) return a;
 	 return b;
}
 
double FRAND()  {
	return (double) rand()/((double)RAND_MAX + 1);
}
 
double MIN( double a, double b ) {
	if (a<b) return a;
	return b;
}



double POW(double x,double n){
  return exp(n*log(x));
}





double HillR(double x,double thresh,double n)
{
	double r=exp(n*log(x/thresh));
	return 1.0/(1+r);
}
 
 
double HillA(double x,double thresh,double n)
{
	double r=exp(n*log(x/thresh));
	return r/(1+r);
 }

 /* Function to compute diffusion of ligands.Geometry encodes the neighbouring cells, i.e. 
geometry[i][] contains the indexes of the neighbours of cell i.  The table of diffusion
csts, difflig[] is static global variable in header
*/

void diffusion(int ncell, int step, double ds[],double history[][NSTEP][NCELLTOT], int geometry[][NNEIGHBOR]){

  int g,neig,index,index_diff,n_localneig;
  double diff,diffusion;
 
 
  for (g=0;g<NDIFFUSIBLE;g++){
 
      diffusion=0;
      index_diff=trackdiff[g];//tracks the diffusible species
      diff=diff_constant[g];//takes the corresponding diffusion constant
      n_localneig=0;//computes number of local neighbours
      for (neig=0;neig<NNEIGHBOR;neig++){
	index=geometry[ncell][neig];//takes the neighoubring cell
	if (index>=0){
	  diffusion+=history[index_diff][step][index];//concentration of the ligand in the neighbouring cell
	  n_localneig+=1;
	}
      }
      diffusion-=n_localneig*history[index_diff][step][ncell];//minus number of local neighbours times concentration of the ligand in the local cell
      diffusion*=diff;// times diffusion constant
      ds[index_diff]+=diffusion;
    }
  }


/* 
computation of the ligand concentration seen by each cells
*/




void sum_concentration(int ncell, int step, double concentrations[]) {

  int l,g,neig,index;
  // compute local ligands concentrations, returns the table "concentrations" 
  for (l=0;l<NLIGAND;l++){
    
  if (externallig[l]==1){
      concentrations[tracklig[l]]=history[tracklig[l]][step][ncell];   // For external ligands, we simply take the local external value as local ligand concentration
  }
  else
    {//in that case, we sum ligand concentration over all neighbouring cells
      g=tracklig[l];
      concentrations[g]=0;
      for (neig=0;neig<NNEIGHBOR;neig++){
	index=geometry[ncell][neig];
	if ((index>=0) && (index!=ncell) ) {//note that we exclude the local concentration of ligand from the sum
	  concentrations[g]+=history[g][step][index];  
	}
      }


    }

  }



}






/* print history array to file= BUFFER, where int try is 0,1,..NTRIES-1 */

void print_history( int try )  {

    int pas, i, j;
    char titre[50];
    FILE *fileptr;

    sprintf(titre, "Buffer%i", try);
    fileptr=fopen(titre, "w");

    for(pas=0; pas<NSTEP; pas++)  {
	fprintf(fileptr,"%i", pas);
	for(j=0; j<NCELLTOT; j++)  {
	    for(i=0; i<SIZE; i++)  {
		fprintf(fileptr,"\t%f",history[i][pas][j]);
	    }
	}
	fprintf(fileptr,"\n");
    }
    fprintf(fileptr,"\n");
    fclose(fileptr);
}



/* statistical tools*/

/* Function that averages the fitness scores*/
double average_score(double score[]){

  double average=0;
  int k;
  for (k=0;k<NTRIES;k++)
    average+=score[k];

  return average/NTRIES;


}


/* Function that computes the variance of fitness scores.*/
double variance_score(double score[]){

  double var=0;
  int k;
  for (k=0;k<NTRIES;k++)
    var+=score[k]*score[k];
  
  var/=NTRIES;
  double average=average_score(score);
  return var-average*average;


}



/* Function that computes the std_deviation*/
double std_dev(double score[]){

  double var=0;
  int k;
  for (k=0;k<NTRIES;k++)
    var+=score[k]*score[k];

  var/=NTRIES;
  double average=average_score(score);
  return sqrt(var-average*average);


}




double gaussdev()
{/*computes a normally distributed deviate with zero mean and unit variance
   using  Box-Muller method, Numerical Recipes C++ p293 */
	static int iset=0;
	static double gset;
	double fac,rsq,v1,v2;

	if (iset == 0) {
	  do {
	    v1=2.0*FRAND()-1.0;
	    v2=2.0*FRAND()-1.0;
	    rsq=v1*v1+v2*v2;
	  } while (rsq >= 1.0 || rsq == 0.0);
	  fac=sqrt(-2.0*log(rsq)/rsq);
	  gset=v1*fac;
	  iset=1;//set flag
	  return v2*fac;//returns one and keep other for nex time
	} 
	else {
	  iset=0;
	  return gset;
	}
}





double compute_noisy_increment(double rate)
{/*computes the increment to add to a ds*/

return rate+gaussdev()*sqrt(rate/(DT*CONCENTRATION_SCALE));


}
 /***** end of header, begiining of python computed functions ***/

 void derivC(double s[],double history[][NSTEP][NCELLTOT],int step, double ds[],double memories[],int ncell){
 int index;	 for (index=0;index<SIZE;index++) ds[index]=0;//initialization
	 double increment=0;
	 double rate=0;

/**************degradation rates*****************/
	 	 rate=1.000000*s[0];
	 	 increment=rate;
	 	 ds[0]-=increment;
	 	 rate=1.000000*s[1];
	 	 increment=rate;
	 	 ds[1]-=increment;
	 	 rate=1.000000*s[2];
	 	 increment=rate;
	 	 ds[2]-=increment;
	 	 rate=1.000000*s[3];
	 	 increment=rate;
	 	 ds[3]-=increment;
	 	 rate=1.000000*s[4];
	 	 increment=rate;
	 	 ds[4]-=increment;
	 	 rate=1.000000*s[5];
	 	 increment=rate;
	 	 ds[5]-=increment;
	 	 rate=1.000000*s[6];
	 	 increment=rate;
	 	 ds[6]-=increment;
	 	 rate=1.000000*s[7];
	 	 increment=rate;
	 	 ds[7]-=increment;
	 	 rate=1.000000*s[8];
	 	 increment=rate;
	 	 ds[8]-=increment;
	 	 rate=1.000000*s[9];
	 	 increment=rate;
	 	 ds[9]-=increment;
	 	 rate=1.000000*s[10];
	 	 increment=rate;
	 	 ds[10]-=increment;
	 	 rate=1.000000*s[11];
	 	 increment=rate;
	 	 ds[11]-=increment;
	 	 rate=1.000000*s[12];
	 	 increment=rate;
	 	 ds[12]-=increment;
	 	 rate=1.000000*s[13];
	 	 increment=rate;
	 	 ds[13]-=increment;
	 	 rate=1.000000*s[14];
	 	 increment=rate;
	 	 ds[14]-=increment;
	 	 rate=1.000000*s[15];
	 	 increment=rate;
	 	 ds[15]-=increment;
	 	 rate=1.000000*s[16];
	 	 increment=rate;
	 	 ds[16]-=increment;
	 	 rate=1.000000*s[17];
	 	 increment=rate;
	 	 ds[17]-=increment;
	 	 rate=1.000000*s[18];
	 	 increment=rate;
	 	 ds[18]-=increment;
	 	 rate=1.000000*s[19];
	 	 increment=rate;
	 	 ds[19]-=increment;
	 	 rate=1.000000*s[20];
	 	 increment=rate;
	 	 ds[20]-=increment;

/**************Transcription rates*****************/
 	 int k,memory=-1;
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.405177,0.000000)*HillR(history[0][memory][ncell],0.388710,4.069446);
	 	 increment=rate;
	 	 ds[1]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.932187*MAX(HillA(history[0][memory][ncell],0.500000,9.000000),HillA(history[6][memory][ncell],0.970203,1.299640)),0.000000)*HillR(history[7][memory][ncell],0.348913,5.619513);
	 	 increment=rate;
	 	 ds[2]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.144473*MAX(HillA(history[0][memory][ncell],0.700000,10.000000),HillA(history[1][memory][ncell],1.573327,3.049788)),0.000000)*HillR(history[4][memory][ncell],0.948909,5.518812)*HillR(history[6][memory][ncell],0.100000,5.629079);
	 	 increment=rate;
	 	 ds[3]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.000992*HillA(history[0][memory][ncell],0.350000,10.000000),0.000000)*HillR(history[2][memory][ncell],0.758037,4.896631)*HillR(history[3][memory][ncell],0.122919,1.000000);
	 	 increment=rate;
	 	 ds[4]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.000000*HillA(history[0][memory][ncell],0.238469,8.481023),0.000000)*HillR(history[4][memory][ncell],0.650000,6.000000)*HillR(history[2][memory][ncell],0.100000,1.985454)*HillR(history[6][memory][ncell],0.180000,5.562659);
	 	 increment=rate;
	 	 ds[5]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.603364*HillA(history[0][memory][ncell],0.486492,5.099465),0.000000)*HillR(history[3][memory][ncell],0.250000,9.663324)*HillR(history[4][memory][ncell],0.334978,7.024320);
	 	 increment=rate;
	 	 ds[8]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.989540,0.000000)*HillR(history[2][memory][ncell],0.430000,10.000000)*HillR(history[6][memory][ncell],0.343886,10.000000)*HillR(history[5][memory][ncell],0.024000,10.000000);
	 	 increment=rate;
	 	 ds[9]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.772335,0.000000)*HillR(history[5][memory][ncell],0.720000,10.000000)*HillR(history[2][memory][ncell],0.085000,6.000000)*HillR(history[6][memory][ncell],0.733272,8.000000);
	 	 increment=rate;
	 	 ds[10]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.116509*HillA(history[0][memory][ncell],0.678114,3.046410),0.000000)*HillR(history[3][memory][ncell],0.100000,8.342839)*HillR(history[6][memory][ncell],0.227239,6.505881)*HillR(history[4][memory][ncell],0.246160,4.558594);
	 	 increment=rate;
	 	 ds[11]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.116509*HillA(history[0][memory][ncell],0.970033,2.711389),0.000000)*HillR(history[2][memory][ncell],1.389836,10.000000)*HillR(history[6][memory][ncell],1.631207,3.276777)*HillR(history[5][memory][ncell],0.240375,2.356542);
	 	 increment=rate;
	 	 ds[12]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.062010*HillA(history[0][memory][ncell],0.382899,10.000000),0.000000)*HillR(history[6][memory][ncell],0.126118,5.445914)*HillR(history[5][memory][ncell],0.200000,2.000000)*HillR(history[3][memory][ncell],0.100000,7.000000)*HillR(history[4][memory][ncell],0.700000,4.000000);
	 	 increment=rate;
	 	 ds[13]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.389365*HillA(history[0][memory][ncell],0.107727,6.331312),0.000000)*HillR(history[4][memory][ncell],0.110000,10.000000)*HillR(history[6][memory][ncell],0.100000,5.932318)*HillR(history[3][memory][ncell],0.540000,10.000000);
	 	 increment=rate;
	 	 ds[14]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.000000*HillA(history[0][memory][ncell],0.600000,10.000000),0.000000)*HillR(history[3][memory][ncell],0.715000,10.000000)*HillR(history[4][memory][ncell],0.042800,10.000000);
	 	 increment=rate;
	 	 ds[15]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.974081*MAX(MAX(MAX(MAX(HillA(history[11][memory][ncell],0.829504,4.850737),HillA(history[15][memory][ncell],0.150800,8.000000)),HillA(history[9][memory][ncell],0.658752,4.856283)),HillA(history[8][memory][ncell],0.831837,8.656311)),HillA(history[10][memory][ncell],0.544538,8.210695)),0.000000);
	 	 increment=rate;
	 	 ds[16]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.900101*HillA(history[7][memory][ncell],0.010000,10.000000),0.000000)*HillR(history[16][memory][ncell],0.200000,10.000000)*HillR(history[2][memory][ncell],1.333853,8.341239)*HillR(history[6][memory][ncell],1.200000,10.000000);
	 	 increment=rate;
	 	 ds[17]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(0.860266*MAX(MAX(MAX(HillA(history[12][memory][ncell],1.092723,1.586230),HillA(history[13][memory][ncell],0.500000,4.000000)),HillA(history[14][memory][ncell],0.170000,3.000000)),HillA(history[17][memory][ncell],0.770000,10.000000)),0.000000);
	 	 increment=rate;
	 	 ds[18]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.000000,0.000000)*HillR(history[6][memory][ncell],1.070000,30.000000)*HillR(history[20][memory][ncell],0.170000,8.000000)*HillR(history[16][memory][ncell],0.350000,8.000000)*HillR(history[18][memory][ncell],0.460000,8.000000)*HillR(history[0][memory][ncell],0.760000,30.000000);
	 	 increment=rate;
	 	 ds[19]+=increment;
	}
	 memory=step-0;
	 if(memory>=0){
	 	 rate=MAX(1.000000,0.000000)*HillR(history[6][memory][ncell],1.070000,30.000000)*HillR(history[16][memory][ncell],0.460000,8.000000)*HillR(history[19][memory][ncell],0.170000,8.000000)*HillR(history[18][memory][ncell],0.350000,8.000000)*HillR(history[0][memory][ncell],0.760000,30.000000);
	 	 increment=rate;
	 	 ds[20]+=increment;
	}

/**************Protein protein interactions*****************/

/**************Phosphorylation*****************/
 float total;
}

void LRinC(double s[],double ds[],double ligands[]){
    double increment=0;
    double rate=0;
}
 
 /***** end of python computed functions, beginning problem specific fns ***/

 #define NFUNCTIONS 3
#define HIGHTHRESH 0.4
#define LOWTHRESH 0.2

static double result[NTRIES][NFUNCTIONS];

//----------------------------Dot Product--------------------------------

double scalar_product(double a[], double b[]){
	int i;
	double scalar = 0;
	
	for (i=0; i<NCELLTOT; i++){
		scalar += a[i]*b[i];
	}
	
	return scalar;
}

//--------------------------------Mean-----------------------------------

double mean(double array[], int n){
	 double sum = 0.0;
	 int i;
	 
	 for ( i=0; i<n; i++){
	 	sum += array[i];
	 }
	 
	 double mean = sum/n;
	 
	 return mean;	 
}

//--------------------------Standard Deviation---------------------------

double standard_dev(double array[], int n){
	double std = 0.0;
	int i;
	
	for (i=0; i<n; i++){
		std += pow((array[i] - mean(array, n)),2);
	}
	std = std/n;
	
	return std;
	
}

//------------------------Count number of stripes-------------------------

int compute_stripe(double a[]){
	int i;
	int nup=0;
	int ndown=0;
	int ncell;
	int test_down,test_up;
	for (ncell=3;ncell<NCELLTOT;ncell++)
    {
		if (a[ncell]<LOWTHRESH)
			test_down=1;
		if ((a[ncell]>HIGHTHRESH)&&(a[ncell-3]<LOWTHRESH)&&(test_down==1))
		{nup+=1;
			test_down=0;
		}
		
		if (a[ncell]>HIGHTHRESH)
			test_up=1;
		if ((a[ncell]<LOWTHRESH)&&(a[ncell-3]>HIGHTHRESH)&&(test_up==1))
		{ndown+=1;
			test_up=0;
		}	
    }
	
	
	//printf("%i %i\n",nup,ndown);
	return (nup+ndown)/2;
}

//------------------------Make Array from .txt file------------------------

void read_concentrations(char name[],double array_concentrations[]){
	
	FILE *fi=NULL;
	int i;
	float limit;
	char line[50];
	i=0;
	fi = fopen(name, "r");
	
	while (fgets(line,NCELLTOT,fi)!=NULL)
    {
		sscanf(line,"%f", &limit);
		array_concentrations[i] = limit;
		i++;	
    }
    
	fclose(fi);
}

//----------------------Write array to .txt file----------------------------

void write2file( char name[], double array[], int n){
	int i;
	
	FILE *fp;
	fp = fopen( name, "w");
	for (i=0; i<n; i++){
		fprintf(fp, "%f\n", array[i]);
	}
	
	fclose(fp);
}

//------------------------------Alternating----------------------------------

double altern(double a[], double b[]){     // Output -1.0 if alternating (a before b), 1.0 if not
    int i;
    int j=0;
    int j1=0;
    double threshdown=0.3;
    int nup=0;
    int nup1=0;
    int ncell;
    int START=3;
    int startup=0;
    int startup1=0;
    int lstripea[10]; //size 10, so max 10 stripes, shouldn't go above 7 anyway
    int lstripeb[10];
    int rstripea[10];
    int rstripeb[10];
    
    // nup determines whether or not we were previously in a section of the cell that is above the threshdown. nup==1 above the threshdown, nup==0 below the end of a stripe and append the edges (startup is where the stripe starts)
    
    for (ncell=START;ncell<NCELLTOT;ncell++){
        //start of a stripe
        if ((a[ncell]>threshdown)&&(nup==0)){
            startup=ncell;
            nup=1;                   
        }
           
        if ((a[ncell]<threshdown)&&(nup==1)){
        	lstripea[j]=startup;
        	rstripea[j]=ncell-1;
            nup=0;
            j +=1;
        }
    }

    //Same but for second gene type b[]

    for (ncell=START;ncell<NCELLTOT;ncell++){
        
        if ((b[ncell]>threshdown)&&(nup1==0)){
            startup1=ncell;
            nup1=1;        
        }        
        
        if ((b[ncell]<threshdown)&&(nup1==1)){ 
        	lstripeb[j1]=startup1;
        	rstripeb[j1]=ncell-1;     
            nup1=0;
            j1 +=1;
        }
    }
    
    int nstripe=compute_stripe(a);
        for (i=0;i<nstripe;i++){
    	//printf("%i %i %i %i\n",lstripea[i],rstripea[i],lstripeb[i],rstripeb[i]);
            if (rstripea[i]>lstripeb[i]){
                return 1.0;
            }
        }
    
        for (i=0;i<nstripe-1;i++){
            if (rstripeb[i]>lstripea[i+1]){
                return 1.0;
            }
        }
            
        return -1.0;
}

//-------------------------------Fitness-------------------------------------

void fitness( double history[][NSTEP][NCELLTOT], int trackout[], int ntry){
	
	
		
}


//--------------------------Treat Fitness------------------------------------


void treatment_fitness( double history2[][NSTEP][NCELLTOT], int trackout[]){
	
	int i,k,l;
	double stat_score[NTRIES];
	double score;
	
	for (k=0; k<NTRIES; k++){
		stat_score[k]=result[k][0];
	}
	
	if (score < RAND_MAX){
	printf("%f\n%f\n%f\n%f \n",result[0][0]+result[0][1]+result[0][2],result[0][0],result[0][1],result[0][2]);
		//printf("%f\n%f\n%f\n",score,average,variance_score);     
	}
	else
		printf("%i\n%i\n%i\n",RAND_MAX,RAND_MAX,RAND_MAX);
	
	/*for (l=1;l<NFUNCTIONS;l++){
		for (k=0; k<NTRIES; k++){
			stat_score[k]=result[k][l];
		}
		score=average_score(stat_score);
		if (l<NFUNCTIONS-1)
			printf("%f\n",score);
		else
			printf("%f",score);
	}*/
}
 /* Define geometry[NCELLTOT][NNEIGHBOR] array defining the neighbors of each cell
   Convention geometry < 0 ie invalid index -> no neighbor, ie used for end cells
   NB even for NCELLTOT=1, should have NNEIGHBOR == 3 to avoid odd seg faults since
   3 neighbors accessed in various loops, eventhough does nothing.
*/
   
void init_geometry() {

  int index;

  for (index=0;index<NCELLTOT;index++){
    geometry[index][0]=index-1;//NB no left neighbour for cell 0 (-1)
    geometry[index][1]=index;
    geometry[index][2]=index+1;
  }

  geometry[NCELLTOT-1][2]=-1;//NB no right neighbour for cell ncell-1 (-1)

}


 
/* set history of all species at initial time to 0  */

void init_history()  {
    int ncell,n_gene;
    for (ncell=0;ncell<NCELLTOT;ncell++){
    	for (n_gene=0;n_gene<SIZE;n_gene++){
      	    history[n_gene][0][ncell]=0;
        }
    }
}

 /* define input variables as a function of time.  n-attempts = loop [0,ntries)  */



static double variability=0;

double r2()
{
    return (double)rand() / (double)RAND_MAX ;
}
	
void inputs(int pas,int ncell, int ntry){
/*if ((pas==0)&&(ncell==0))
   {variability=0.2*FRAND();
   }
*/
	
	int i=0;
	int j=0;
	for (j=1;j<21;j++){
	    if (pas>=300+25*(j-1) && pas<300+25*j){
	        i=j;
	        break;
	        }
	    }
	if (pas>=700){
		i=20;
	}
	/*
	else if (pas>=600 && pas<700){
		i=16;
	}
	else if (pas>=500 && pas<600){
		i=12;
	}
	else if (pas>=400 && pas<500){
		i=8;
	}
	else if (pas>=300 && pas<400){
		i=4;
	}
	*/
	double bcd=exp(0.01*(NCELLTOT-(ncell+i)))/7;
	double tll=exp(0.03*(ncell+i)-6);
	double hkb=exp(0.04*(ncell+i)-8);
	
  
	history[trackin[0]][pas][ncell]=bcd;
 
	history[trackin[1]][pas][ncell]=tll;
	
	history[trackin[2]][pas][ncell]=hkb;
	
	double x = 0.2*r2();
	double y = 0.2*r2();
	double z = 0.2*r2();
	
	for (j=1;j<300;j++){
	    history[18][j][ncell]=x;
	    history[19][j][ncell]=y;
	    history[20][j][ncell]=z;
	    }
	    

}


 
/* compute the RHS of equations and run over NSTEP's with 1st order Euler method
   The arugment kk, is an index passed to inputs that records which iteration of 
   initial or boundary conditions the same system of equs is being integrated for

   This version of code used with ligand at cell computed from sum neighbors
*/
 
void integrator(int kk){

    double s[SIZE];
    double ds[SIZE]; 
    double sumligands[SIZE];
    double memory[SIZE];
    int index,n,pas,ncell;

    for (index=0;index<SIZE;index++){
	s[index] = 0;
        ds[index]=0;
        memory[index]=0;
    }
 
    /* initialize geometry here, incase cells move  */
    init_geometry();
    init_history();

    /* loop over time steps, then over each cell etc */
    for (pas=0;pas<NSTEP-1;pas++)  {
	for (ncell=0;ncell<NCELLTOT;ncell++)  {
            inputs(pas,ncell,kk);
            for (index=0;index<SIZE;index++) {
	        s[index]=history[index][pas][ncell];
            }
            derivC(s,history,pas,ds,memory,ncell);  //local integration
            sum_concentration(ncell,pas,sumligands);  //perform sum of ligands concentrations for non external ligands
	    diffusion(ncell,pas,ds,history,geometry);//computes diffusion of external ligands
            LRinC(s,ds,sumligands);
	   
            for (index=0;index<SIZE;index++) {
	 	 history[index][pas+1][ncell] = s[index] + DT*ds[index];
		 if (history[index][pas+1][ncell]<0)//might happen for langevin
		   history[index][pas+1][ncell]=0;
	    }
	}
    }

    /* fill in inputs for last time.  */
    for (ncell=0;ncell<NCELLTOT;ncell++)  {
      inputs(NSTEP-1,ncell,kk);
    }
}

 

/*  General main.c, should not need any others, see comments in fitness_template for 
the functions that must be supplied in that file for the main to work,
*/

int main()  {
    srand( SEED );
    int i, k,l;
    double score = 0;
    int len_hist = SIZE*NCELLTOT*NSTEP;
    double *hptr = &history[0][0][0];
    /* following incase one wants to average history before doing fitness */
    static double history2[SIZE][NSTEP][NCELLTOT];
    double *h2ptr = &history2[0][0][0];//table for averaging output (used for multicell problems)

   

   
    /* dummy return when no outputs for fitness function */

    if(NOUTPUT <= 0) {
    	printf("%s","no output variables? terminating without integration" );
	//return;
    }
    
    for(i=0; i<len_hist; i++)  {
	*(h2ptr + i) = 0;
    }
    

    for (k=0; k<NTRIES; k++){
    	integrator(k);
	fitness(history, trackout,k);
       	if( PRINT_BUF )  {
	    print_history(k);
	}

	for(i=0; i<len_hist; i++)  {
	    *(h2ptr+i) += *(hptr +i);
	}
    }


    treatment_fitness(history2,trackout);

  
}  

